# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pymisp',
 'pymisp.data.misp-objects',
 'pymisp.data.misp-objects.tools',
 'pymisp.tools']

package_data = \
{'': ['*'],
 'pymisp': ['data/*'],
 'pymisp.data.misp-objects': ['.github/workflows/*',
                              'docs/*',
                              'objects/ail-leak/*',
                              'objects/ais-info/*',
                              'objects/android-app/*',
                              'objects/android-permission/*',
                              'objects/annotation/*',
                              'objects/anonymisation/*',
                              'objects/apivoid-email-verification/*',
                              'objects/artifact/*',
                              'objects/asn/*',
                              'objects/attack-pattern/*',
                              'objects/attack-step/*',
                              'objects/authentication-failure-report/*',
                              'objects/authenticode-signerinfo/*',
                              'objects/av-signature/*',
                              'objects/bank-account/*',
                              'objects/bgp-hijack/*',
                              'objects/bgp-ranking/*',
                              'objects/blog/*',
                              'objects/boleto/*',
                              'objects/btc-transaction/*',
                              'objects/btc-wallet/*',
                              'objects/cap-alert/*',
                              'objects/cap-info/*',
                              'objects/cap-resource/*',
                              'objects/cloth/*',
                              'objects/coin-address/*',
                              'objects/command-line/*',
                              'objects/command/*',
                              'objects/concordia-mtmf-intrusion-set/*',
                              'objects/cookie/*',
                              'objects/cortex-taxonomy/*',
                              'objects/cortex/*',
                              'objects/course-of-action/*',
                              'objects/covid19-csse-daily-report/*',
                              'objects/covid19-dxy-live-city/*',
                              'objects/covid19-dxy-live-province/*',
                              'objects/cowrie/*',
                              'objects/cpe-asset/*',
                              'objects/credential/*',
                              'objects/credit-card/*',
                              'objects/crypto-material/*',
                              'objects/cytomic-orion-file/*',
                              'objects/cytomic-orion-machine/*',
                              'objects/dark-pattern-item/*',
                              'objects/ddos/*',
                              'objects/device/*',
                              'objects/diameter-attack/*',
                              'objects/dkim/*',
                              'objects/dns-record/*',
                              'objects/domain-crawled/*',
                              'objects/domain-ip/*',
                              'objects/edr-report/*',
                              'objects/elf-section/*',
                              'objects/elf/*',
                              'objects/email/*',
                              'objects/employee/*',
                              'objects/error-message/*',
                              'objects/exploit-poc/*',
                              'objects/exploit/*',
                              'objects/facebook-account/*',
                              'objects/facebook-group/*',
                              'objects/facebook-page/*',
                              'objects/facebook-post/*',
                              'objects/facebook-reaction/*',
                              'objects/facial-composite/*',
                              'objects/fail2ban/*',
                              'objects/favicon/*',
                              'objects/file/*',
                              'objects/forensic-case/*',
                              'objects/forensic-evidence/*',
                              'objects/forged-document/*',
                              'objects/ftm-Airplane/*',
                              'objects/ftm-Assessment/*',
                              'objects/ftm-Asset/*',
                              'objects/ftm-Associate/*',
                              'objects/ftm-Audio/*',
                              'objects/ftm-BankAccount/*',
                              'objects/ftm-Call/*',
                              'objects/ftm-Company/*',
                              'objects/ftm-Contract/*',
                              'objects/ftm-ContractAward/*',
                              'objects/ftm-CourtCase/*',
                              'objects/ftm-CourtCaseParty/*',
                              'objects/ftm-Debt/*',
                              'objects/ftm-Directorship/*',
                              'objects/ftm-Document/*',
                              'objects/ftm-Documentation/*',
                              'objects/ftm-EconomicActivity/*',
                              'objects/ftm-Email/*',
                              'objects/ftm-Event/*',
                              'objects/ftm-Family/*',
                              'objects/ftm-Folder/*',
                              'objects/ftm-HyperText/*',
                              'objects/ftm-Image/*',
                              'objects/ftm-Land/*',
                              'objects/ftm-LegalEntity/*',
                              'objects/ftm-License/*',
                              'objects/ftm-Membership/*',
                              'objects/ftm-Message/*',
                              'objects/ftm-Organization/*',
                              'objects/ftm-Ownership/*',
                              'objects/ftm-Package/*',
                              'objects/ftm-Page/*',
                              'objects/ftm-Pages/*',
                              'objects/ftm-Passport/*',
                              'objects/ftm-Payment/*',
                              'objects/ftm-Person/*',
                              'objects/ftm-PlainText/*',
                              'objects/ftm-PublicBody/*',
                              'objects/ftm-RealEstate/*',
                              'objects/ftm-Representation/*',
                              'objects/ftm-Row/*',
                              'objects/ftm-Sanction/*',
                              'objects/ftm-Succession/*',
                              'objects/ftm-Table/*',
                              'objects/ftm-TaxRoll/*',
                              'objects/ftm-UnknownLink/*',
                              'objects/ftm-UserAccount/*',
                              'objects/ftm-Vehicle/*',
                              'objects/ftm-Vessel/*',
                              'objects/ftm-Video/*',
                              'objects/ftm-Workbook/*',
                              'objects/game-cheat/*',
                              'objects/geolocation/*',
                              'objects/git-vuln-finder/*',
                              'objects/github-user/*',
                              'objects/gitlab-user/*',
                              'objects/gtp-attack/*',
                              'objects/hashlookup/*',
                              'objects/http-request/*',
                              'objects/identity/*',
                              'objects/ilr-impact/*',
                              'objects/ilr-notification-incident/*',
                              'objects/image/*',
                              'objects/impersonation/*',
                              'objects/imsi-catcher/*',
                              'objects/infrastructure/*',
                              'objects/instant-message-group/*',
                              'objects/instant-message/*',
                              'objects/intel471-vulnerability-intelligence/*',
                              'objects/intelmq_event/*',
                              'objects/intelmq_report/*',
                              'objects/internal-reference/*',
                              'objects/interpol-notice/*',
                              'objects/intrusion-set/*',
                              'objects/iot-device/*',
                              'objects/iot-firmware/*',
                              'objects/ip-api-address/*',
                              'objects/ip-port/*',
                              'objects/irc/*',
                              'objects/ja3/*',
                              'objects/ja3s/*',
                              'objects/jarm/*',
                              'objects/keybase-account/*',
                              'objects/language-content/*',
                              'objects/leaked-document/*',
                              'objects/legal-entity/*',
                              'objects/lnk/*',
                              'objects/macho-section/*',
                              'objects/macho/*',
                              'objects/mactime-timeline-analysis/*',
                              'objects/malware-config/*',
                              'objects/meme-image/*',
                              'objects/microblog/*',
                              'objects/mutex/*',
                              'objects/narrative/*',
                              'objects/netflow/*',
                              'objects/network-connection/*',
                              'objects/network-profile/*',
                              'objects/network-socket/*',
                              'objects/news-agency/*',
                              'objects/news-media/*',
                              'objects/open-data-security/*',
                              'objects/organization/*',
                              'objects/original-imported-file/*',
                              'objects/paloalto-threat-event/*',
                              'objects/parler-account/*',
                              'objects/parler-comment/*',
                              'objects/parler-post/*',
                              'objects/passive-dns-dnsdbflex/*',
                              'objects/passive-dns/*',
                              'objects/passive-ssh/*',
                              'objects/paste/*',
                              'objects/pcap-metadata/*',
                              'objects/pe-section/*',
                              'objects/pe/*',
                              'objects/person/*',
                              'objects/personification/*',
                              'objects/pgp-meta/*',
                              'objects/phishing-kit/*',
                              'objects/phishing/*',
                              'objects/phone/*',
                              'objects/postal-address/*',
                              'objects/probabilistic-data-structure/*',
                              'objects/process/*',
                              'objects/publication/*',
                              'objects/python-etvx-event-log/*',
                              'objects/query/*',
                              'objects/r2graphity/*',
                              'objects/ransom-negotiation/*',
                              'objects/reddit-account/*',
                              'objects/reddit-comment/*',
                              'objects/reddit-post/*',
                              'objects/reddit-subreddit/*',
                              'objects/regexp/*',
                              'objects/registry-key/*',
                              'objects/regripper-NTUser/*',
                              'objects/regripper-sam-hive-single-user/*',
                              'objects/regripper-sam-hive-user-group/*',
                              'objects/regripper-software-hive-BHO/*',
                              'objects/regripper-software-hive-appInit-DLLS/*',
                              'objects/regripper-software-hive-application-paths/*',
                              'objects/regripper-software-hive-applications-installed/*',
                              'objects/regripper-software-hive-command-shell/*',
                              'objects/regripper-software-hive-software-run/*',
                              'objects/regripper-software-hive-userprofile-winlogon/*',
                              'objects/regripper-software-hive-windows-general-info/*',
                              'objects/regripper-system-hive-firewall-configuration/*',
                              'objects/regripper-system-hive-general-configuration/*',
                              'objects/regripper-system-hive-network-information/*',
                              'objects/regripper-system-hive-services-drivers/*',
                              'objects/report/*',
                              'objects/research-scanner/*',
                              'objects/rogue-dns/*',
                              'objects/rtir/*',
                              'objects/sandbox-report/*',
                              'objects/sb-signature/*',
                              'objects/scheduled-event/*',
                              'objects/scheduled-task/*',
                              'objects/scrippsco2-c13-daily/*',
                              'objects/scrippsco2-c13-monthly/*',
                              'objects/scrippsco2-co2-daily/*',
                              'objects/scrippsco2-co2-monthly/*',
                              'objects/scrippsco2-o18-daily/*',
                              'objects/scrippsco2-o18-monthly/*',
                              'objects/script/*',
                              'objects/security-playbook/*',
                              'objects/shell-commands/*',
                              'objects/shodan-report/*',
                              'objects/short-message-service/*',
                              'objects/shortened-link/*',
                              'objects/sigma/*',
                              'objects/social-media-group/*',
                              'objects/software/*',
                              'objects/spearphishing-attachment/*',
                              'objects/spearphishing-link/*',
                              'objects/splunk/*',
                              'objects/ss7-attack/*',
                              'objects/ssh-authorized-keys/*',
                              'objects/stix2-pattern/*',
                              'objects/stock/*',
                              'objects/submarine/*',
                              'objects/suricata/*',
                              'objects/target-system/*',
                              'objects/tattoo/*',
                              'objects/telegram-account/*',
                              'objects/telegram-bot/*',
                              'objects/temporal-event/*',
                              'objects/threatgrid-report/*',
                              'objects/timecode/*',
                              'objects/timesketch-timeline/*',
                              'objects/timesketch_message/*',
                              'objects/timestamp/*',
                              'objects/tor-hiddenservice/*',
                              'objects/tor-node/*',
                              'objects/tracking-id/*',
                              'objects/transaction/*',
                              'objects/translation/*',
                              'objects/trustar_report/*',
                              'objects/tsk-chats/*',
                              'objects/tsk-web-bookmark/*',
                              'objects/tsk-web-cookie/*',
                              'objects/tsk-web-downloads/*',
                              'objects/tsk-web-history/*',
                              'objects/tsk-web-search-query/*',
                              'objects/twitter-account/*',
                              'objects/twitter-list/*',
                              'objects/twitter-post/*',
                              'objects/url/*',
                              'objects/user-account/*',
                              'objects/vehicle/*',
                              'objects/victim/*',
                              'objects/virustotal-graph/*',
                              'objects/virustotal-report/*',
                              'objects/virustotal-submission/*',
                              'objects/vulnerability/*',
                              'objects/weakness/*',
                              'objects/whois/*',
                              'objects/windows-service/*',
                              'objects/x509/*',
                              'objects/yabin/*',
                              'objects/yara/*',
                              'objects/youtube-channel/*',
                              'objects/youtube-comment/*',
                              'objects/youtube-playlist/*',
                              'objects/youtube-video/*',
                              'relationships/*'],
 'pymisp.tools': ['pdf_fonts/*']}

install_requires = \
['deprecated>=1.2.13,<2.0.0',
 'jsonschema>=4.17.0,<5.0.0',
 'python-dateutil>=2.8.2,<3.0.0',
 'requests>=2.28.1,<3.0.0']

extras_require = \
{'brotli': ['urllib3[brotli]>=1.26.12,<2.0.0'],
 'docs': ['sphinx-autodoc-typehints>=1.19.5,<2.0.0',
          'recommonmark>=0.7.1,<0.8.0'],
 'email': ['extract_msg>=0.36.4,<0.37.0',
           'RTFDE>=0.0.2,<0.0.3',
           'oletools>=0.60.1,<0.61.0'],
 'fileobjects': ['python-magic>=0.4.27,<0.5.0',
                 'pydeep2>=0.5.1,<0.6.0',
                 'lief>=0.12.3,<0.13.0'],
 'openioc': ['beautifulsoup4>=4.11.1,<5.0.0'],
 'pdfexport': ['reportlab>=3.6.12,<4.0.0'],
 'url': ['pyfaup>=1.2,<2.0', 'chardet>=5.0.0,<6.0.0'],
 'virustotal': ['validators>=0.20.0,<0.21.0']}

setup_kwargs = {
    'name': 'pymisp',
    'version': '2.4.162.2',
    'description': 'Python API for MISP.',
    'long_description': '**IMPORTANT NOTE**: This library will require **at least** python 3.8 starting the 1st of January 2022. If you have legacy versions of python, please use the latest PyMISP version that will be released in December 2021, and consider updating your system(s). Anything released within the last 2 years will do, starting with Ubuntu 20.04.\n\n# PyMISP - Python Library to access MISP\n\n[![Documentation Status](https://readthedocs.org/projects/pymisp/badge/?version=latest)](http://pymisp.readthedocs.io/?badge=latest)\n[![Coverage Status](https://coveralls.io/repos/github/MISP/PyMISP/badge.svg?branch=main)](https://coveralls.io/github/MISP/PyMISP?branch=main)\n[![Python 3.6](https://img.shields.io/badge/python-3.6+-blue.svg)](https://www.python.org/downloads/release/python-360/)\n[![PyPi version](https://img.shields.io/pypi/v/pymisp.svg)](https://pypi.python.org/pypi/pymisp/)\n[![Number of PyPI downloads](https://img.shields.io/pypi/dm/pymisp.svg)](https://pypi.python.org/pypi/pymisp/)\n\nPyMISP is a Python library to access [MISP](https://github.com/MISP/MISP) platforms via their REST API.\n\nPyMISP allows you to fetch events, add or update events/attributes, add or update samples or search for attributes.\n\n## Install from pip\n\n**It is strongly recommended to use a virtual environment**\n\nIf you want to know more about virtual environments, [python has you covered](https://docs.python.org/3/tutorial/venv.html)\n\nOnly basic dependencies:\n```\npip3 install pymisp\n```\n\nAnd there are a few optional dependencies:\n* fileobjects: to create PE/ELF/Mach-o objects\n* openioc: to import files in OpenIOC format (not really maintained)\n* virustotal: to query VirusTotal and generate the appropriate objects\n* docs: to generate te documentation\n* pdfexport: to generate PDF reports out of MISP events\n* url: to generate URL objects out of URLs with Pyfaup\n* email: to generate MISP Email objects\n* brotli: to use the brotli compression when interacting with a MISP instance\n\nExample: \n\n```\npip3 install pymisp[virustotal,email]\n```\n\n## Install the latest version from repo from development purposes\n\n**Note**: poetry is required; e.g., "pip3 install poetry"\n\n```\ngit clone https://github.com/MISP/PyMISP.git && cd PyMISP\ngit submodule update --init\npoetry install -E fileobjects -E openioc -E virustotal -E docs -E pdfexport\n```\n\n### Running the tests\n\n```bash\npoetry run nosetests-3.4 --with-coverage --cover-package=pymisp,tests --cover-tests tests/test_*.py\n```\n\nIf you have a MISP instance to test against, you can also run the live ones:\n\n**Note**: You need to update the key in `tests/testlive_comprehensive.py` to the automation key of your admin account.\n\n```bash\npoetry run nosetests-3.4 --with-coverage --cover-package=pymisp,tests --cover-tests tests/testlive_comprehensive.py\n```\n\n## Samples and how to use PyMISP\n\nVarious examples and samples scripts are in the [examples/](examples/) directory.\n\nIn the examples directory, you will need to change the keys.py.sample to enter your MISP url and API key.\n\n```\ncd examples\ncp keys.py.sample keys.py\nvim keys.py\n```\n\nThe API key of MISP is available in the Automation section of the MISP web interface.\n\nTo test if your URL and API keys are correct, you can test with examples/last.py to\nfetch the events published in the last x amount of time (supported time indicators: days (d), hours (h) and minutes (m)).\nlast.py\n```\ncd examples\npython3 last.py -l 10h # 10 hours\npython3 last.py -l 5d  #  5 days\npython3 last.py -l 45m # 45 minutes\n```\n\n\n## Debugging\n\nYou have two options here:\n\n1. Pass `debug=True` to `PyMISP` and it will enable logging.DEBUG to stderr on the whole module\n\n2. Use the python logging module directly:\n\n```python\n\nimport logging\nlogger = logging.getLogger(\'pymisp\')\n\n# Configure it as you wish, for example, enable DEBUG mode:\nlogger.setLevel(logging.DEBUG)\n```\n\nOr if you want to write the debug output to a file instead of stderr:\n\n```python\nimport pymisp\nimport logging\n\nlogger = logging.getLogger(\'pymisp\')\nlogging.basicConfig(level=logging.DEBUG, filename="debug.log", filemode=\'w\', format=pymisp.FORMAT)\n```\n\n## Test cases\n\n1. The content of `mispevent.py` is tested on every commit\n2. The test cases that require a running MISP instance can be run the following way:\n\n\n```bash\n# From poetry\n\nnosetests-3.4 -s --with-coverage --cover-package=pymisp,tests --cover-tests tests/testlive_comprehensive.py:TestComprehensive.[test_name]\n\n```\n\n## Documentation\n\nThe documentation is available [here](https://pymisp.readthedocs.io/en/latest/).\n\n### Jupyter notebook\n\nA series of [Jupyter notebooks for PyMISP tutorial](https://github.com/MISP/PyMISP/tree/main/docs/tutorial) are available in the repository.\n\n## Everything is a Mutable Mapping\n\n... or at least everything that can be imported/exported from/to a json blob\n\n`AbstractMISP` is the master class, and inherits from `collections.MutableMapping` which means\nthe class can be represented as a python dictionary.\n\nThe abstraction assumes every property that should not be seen in the dictionary is prepended with a `_`,\nor its name is added to the private list `__not_jsonable` (accessible through `update_not_jsonable` and `set_not_jsonable`.\n\nThis master class has helpers that make it easy to load, and export to, and from, a json string.\n\n`MISPEvent`, `MISPAttribute`, `MISPObjectReference`, `MISPObjectAttribute`, and `MISPObject`\nare subclasses of AbstractMISP, which mean that they can be handled as python dictionaries.\n\n## MISP Objects\n\nCreating a new MISP object generator should be done using a pre-defined template and inherit `AbstractMISPObjectGenerator`.\n\nYour new MISPObject generator must generate attributes and add them as class properties using `add_attribute`.\n\nWhen the object is sent to MISP, all the class properties will be exported to the JSON export.\n\n# License\n\nPyMISP is distributed under an [open source license](./LICENSE). A simplified 2-BSD license.\n\n',
    'author': 'Raphaël Vinot',
    'author_email': 'raphael.vinot@circl.lu',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/MISP/PyMISP',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import zipfile
from io import BytesIO
from pathlib import Path

import requests

from ..abstract import resources_path

static_repo = "https://github.com/MISP/misp-objects/archive/main.zip"


def update_objects():
    r = requests.get(static_repo)

    zipped_repo = BytesIO(r.content)

    with zipfile.ZipFile(zipped_repo, 'r') as myzip:
        for name in myzip.namelist():
            if not name.endswith('.json'):
                continue
            name_on_disk = name.replace('misp-objects-main', 'misp-objects')
            path = resources_path / Path(name_on_disk)
            if not path.parent.exists():
                path.parent.mkdir(parents=True)
            with path.open('wb') as f:
                f.write(myzip.read(name))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from pathlib import Path
import json

name_ontology = 'alfred'

relationships_path = Path('..', 'relationships', 'definition.json')

with open(relationships_path) as f:
    relationships = json.load(f)

rel_fast_lookup = {entry['name']: entry for entry in relationships['values']}

ontology_path = Path('alfred-ontology.json')

with open(ontology_path) as f:
    ontology = json.load(f)

links = ontology['data']['linkTypes']


for linktype in links:
    link_name = linktype['name'].lower().replace('_', '-')
    link_description = linktype['description']
    if link_name in rel_fast_lookup:
        if rel_fast_lookup[link_name]['description'] != link_description:
            print(link_name)
            print('\t MISP:', rel_fast_lookup[link_name]['description'])
            print('\t Alfred:', link_description)
        for entry in relationships['values']:
            if entry['name'] == link_name:
                if name_ontology not in entry['format']:
                    entry['format'].append(name_ontology)
                break
        # Update the fast lookup to avoid duplicates.
        rel_fast_lookup = {entry['name']: entry for entry in relationships['values']}
    else:
        if link_name not in rel_fast_lookup:
            linktype['name'] = link_name
            linktype['format'] = [name_ontology]
            relationships['values'].append(linktype)
        else:
            print("Duplicate", link_name)

with open(relationships_path, 'w') as f:
    json.dump(relationships, f, indent=2)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# NOTE: Reference on how this module is used: https://vvx7.io/posts/2020/05/misp-slack-bot/

from .abstractgenerator import AbstractMISPObjectGenerator
import logging

logger = logging.getLogger('pymisp')


class MicroblogObject(AbstractMISPObjectGenerator):

    def __init__(self, parameters: dict, strict: bool = True, **kwargs):
        super().__init__('microblog', strict=strict, **kwargs)
        self._parameters = parameters
        self.generate_attributes()

    def generate_attributes(self):
        # Raw post.
        if 'post' in self._parameters:
            self.add_attribute('post', value=self._parameters['post'])

        # Title of the post.
        if 'title' in self._parameters:
            self.add_attribute('title', value=self._parameters['title'])

        # Original link into the microblog post (Supposed harmless).
        if 'link' in self._parameters:
            self.add_attribute('link', value=self._parameters['link'])

        # Original URL location of the microblog post (potentially malicious.
        if 'url' in self._parameters:
            if isinstance(self._parameters.get('url'), list):
                for i in self._parameters.get('url'):
                    self.add_attribute('url', value=i)
            else:
                self.add_attribute('url', value=self._parameters['url'])

        # Archive of the original document (Internet Archive, Archive.is, etc).
        if 'archive' in self._parameters:
            if isinstance(self._parameters.get('archive'), list):
                for i in self._parameters.get('archive'):
                    self.add_attribute('archive', value=i)
            else:
                self.add_attribute('archive', value=self._parameters['archive'])

        # Display name of the account who posted the microblog.
        if 'display-name' in self._parameters:
            self.add_attribute('display-name', value=self._parameters['display-name'])

        # The user ID of the microblog this post replies to.
        if 'in-reply-to-user-id' in self._parameters:
            self.add_attribute('in-reply-to-user-id', value=self._parameters['in-reply-to-user-id'])

        # The microblog ID of the microblog this post replies to.
        if 'in-reply-to-status-id' in self._parameters:
            self.add_attribute('in-reply-to-status-id', value=self._parameters['in-reply-to-status-id'])

        # The user display name of the microblog this post replies to.
        if 'in-reply-to-display-name' in self._parameters:
            self.add_attribute('in-reply-to-display-name', value=self._parameters['in-reply-to-display-name'])

        # The language of the post.
        if 'language' in self._parameters:
            self.add_attribute('language', value=self._parameters['language'], disable_correlation=True)

        # The microblog post file or screen capture.
        # if 'attachment' in self._parameters:
        #     self.add_attribute('attachment', value=self._parameters['attachment'])

        # Type of the microblog post.
        type_allowed_values = ["Twitter", "Facebook", "LinkedIn", "Reddit", "Google+",
                               "Instagram", "Forum", "Other"]
        if 'type' in self._parameters:
            if isinstance(self._parameters.get('type'), list):
                for i in self._parameters.get('type'):
                    if i in type_allowed_values:
                        self.add_attribute('type', value=i)
            else:
                if self._parameters['type'] in type_allowed_values:
                    self.add_attribute('type', value=self._parameters['type'])

        # State of the microblog post.
        type_allowed_values = ["Informative", "Malicious", "Misinformation", "Disinformation", "Unknown"]
        if 'state' in self._parameters:
            if isinstance(self._parameters.get('state'), list):
                for i in self._parameters.get('state'):
                    if i in type_allowed_values:
                        self.add_attribute('state', value=i)
            else:
                if self._parameters['state'] in type_allowed_values:
                    self.add_attribute('state', value=self._parameters['state'])

        # Username who posted the microblog post (without the @ prefix).
        if 'username' in self._parameters:
            self.add_attribute('username', value=self._parameters['username'])

        # == the username account verified by the operator of the microblog platform.
        type_allowed_values = ["Verified", "Unverified", "Unknown"]
        if 'verified-username' in self._parameters:
            if isinstance(self._parameters.get('verified-username'), list):
                for i in self._parameters.get('verified-username'):
                    if i in type_allowed_values:
                        self.add_attribute('verified-username', value=i)
            else:
                if self._parameters['verified-username'] in type_allowed_values:
                    self.add_attribute('verified-username', value=self._parameters['verified-username'])

        # embedded-link.
        if 'embedded-link' in self._parameters:
            if isinstance(self._parameters.get('embedded-link'), list):
                for i in self._parameters.get('embedded-link'):
                    self.add_attribute('embedded-link', value=i)
            else:
                self.add_attribute('embedded-link', value=self._parameters['embedded-link'])

        # embedded-safe-link
        if 'embedded-safe-link' in self._parameters:
            if isinstance(self._parameters.get('embedded-safe-link'), list):
                for i in self._parameters.get('embedded-safe-link'):
                    self.add_attribute('embedded-safe-link', value=i)
            else:
                self.add_attribute('embedded-safe-link', value=self._parameters['embedded-safe-link'])

        # Hashtag into the microblog post.
        if 'hashtag' in self._parameters:
            if isinstance(self._parameters.get('hashtag'), list):
                for i in self._parameters.get('hashtag'):
                    self.add_attribute('hashtag', value=i)
            else:
                self.add_attribute('hashtag', value=self._parameters['hashtag'])

        # username quoted
        if 'username-quoted' in self._parameters:
            if isinstance(self._parameters.get('username-quoted'), list):
                for i in self._parameters.get('username-quoted'):
                    self.add_attribute('username-quoted', value=i)
            else:
                self.add_attribute('username-quoted', value=self._parameters['username-quoted'])

        # twitter post id
        if 'twitter-id' in self._parameters:
            self.add_attribute('twitter-id', value=self._parameters['twitter-id'])




# -*- coding: utf-8 -*-

import os

from .. import MISPEvent
try:
    from bs4 import BeautifulSoup  # type: ignore
    has_bs4 = True
except ImportError:
    has_bs4 = False

iocMispMapping = {
    'CookieHistoryItem/HostName': {'type': 'hostname', 'comment': 'CookieHistory.'},

    'DriverItem/DriverName': {'category': 'Artifacts dropped', 'type': 'other', 'comment': 'DriverName.'},
    'DriverItem/CertificateIssuer': {'category': 'Artifacts dropped', 'type': 'other', 'comment': 'DriverCertificateIssuer.'},
    'DriverItem/DeviceItem/AttachedDeviceName': {'category': 'Artifacts dropped', 'type': 'other', 'comment': 'DriverDeviceName. '},

    'DnsEntryItem/Host': {'type': 'domain'},
    'DnsEntryItem/RecordName': {'type': 'domain'},

    'Email/To': {'type': 'target-email'},
    'Email/Date': {'type': 'comment', 'comment': 'EmailDate.'},
    'Email/Body': {'type': 'email-subject'},
    'Email/From': {'type': 'email-dst'},
    'Email/Subject': {'type': 'email-subject'},
    'Email/Attachment/Name': {'type': 'email-attachment'},

    'FileItem/Md5sum': {'type': 'md5'},
    'FileItem/Sha1sum': {'type': 'sha1'},
    'FileItem/FileName': {'type': 'filename'},
    'FileItem/FullPath': {'type': 'filename'},
    'FileItem/FilePath': {'type': 'filename'},
    'FileItem/Sha256sum': {'type': 'sha256'},
    'FileItem/DevicePath': {'type': 'comment', 'comment': 'DevicePath. '},
    'FileItem/SizeInBytes': {'type': 'size-in-bytes'},
    'FileItem/PEInfo/Type': {'type': 'comment', 'comment': 'PE Type. '},
    'FileItem/FileExtension': {'type': 'comment', 'comment': 'FileExtension. '},
    'FileItem/FilenameCreated': {'type': 'filename', 'comment': 'FilenameCreated. '},
    'FileItem/StringList/string': {'type': 'pattern-in-file', 'comment': 'string list. '},
    'FileItem/PEInfo/PETimeStamp': {'type': 'pattern-in-file', 'comment': 'PE TimeStamp. '},
    'FileItem/PEInfo/Exports/DllName': {'type': 'pattern-in-memory', 'comment': 'PE export DllName. '},
    'FileItem/PEInfo/Sections/Section/Name': {'type': 'pattern-in-memory', 'comment': 'PE SectionName. '},
    'FileItem/PEInfo/DetectedAnomalies/string': {'type': 'pattern-in-file', 'comment': 'PE DEtected AnomaliesString. '},
    'FileItem/PEInfo/Exports/NumberOfFunctions': {'type': 'pattern-in-file', 'comment': 'PE Export NumberOfFunctions. '},
    'FileItem/PEInfo/ImportedModules/Module/Name': {'type': 'pattern-in-file', 'comment': 'PE ImportedModulesName. '},
    'FileItem/PEInfo/DigitalSignature/Description': {'type': 'comment', 'comment': 'PE DigitalSignatureDescription. '},
    'FileItem/PEInfo/DigitalSignature/SignatureExists': {'type': 'comment', 'comment': 'PE SignatureExists. '},
    'FileItem/PEInfo/Exports/ExportedFunctions/string': {'type': 'comment', 'comment': 'PE ExportedFunctions. '},
    'FileItem/PEInfo/DigitalSignature/CertificateIssuer': {'type': 'comment', 'comment': 'PE SignatureCertificateIssuer. '},
    'FileItem/PEInfo/DigitalSignature/SignatureVerified': {'type': 'comment', 'comment': 'PE SignatureVerified. '},
    'FileItem/PEInfo/DigitalSignature/CertificateSubject': {'type': 'other', 'comment': 'PE CertificateDigitalSignatureSubject. '},
    'FileItem/PEInfo/ResourceInfoList/ResourceInfoItem/Name': {'type': 'comment', 'comment': 'PE ResourceName. '},
    'FileItem/PEInfo/ResourceInfoList/ResourceInfoItem/Type': {'type': 'comment', 'comment': 'PE ResourceType. '},
    'FileItem/PEInfo/VersionInfoList/VersionInfoItem/Language': {'type': 'pattern-in-file', 'comment': 'PE LanguageVersion. '},
    'FileItem/PEInfo/ResourceInfoList/ResourceInfoItem/Language': {'type': 'pattern-in-file', 'comment': 'PE LanguageResource. '},
    'FileItem/PEInfo/VersionInfoList/VersionInfoItem/CompanyName': {'type': 'pattern-in-file', 'comment': 'PE versionInfo CompanyName. '},
    'FileItem/PEInfo/VersionInfoList/VersionInfoItem/FileVersion': {'type': 'pattern-in-file', 'comment': 'PE Version. '},
    'FileItem/PEInfo/VersionInfoList/VersionInfoItem/ProductName': {'type': 'pattern-in-file', 'comment': 'PE ProductName. '},
    'FileItem/PEInfo/VersionInfoList/VersionInfoItem/InternalName': {'type': 'pattern-in-file', 'comment': 'PE InternalName. '},
    'FileItem/PEInfo/VersionInfoList/VersionInfoItem/LegalCopyright': {'type': 'pattern-in-file', 'comment': 'PE LegalCopyright. '},
    'FileItem/PEInfo/VersionInfoList/VersionInfoItem/ProductVersion': {'type': 'pattern-in-file', 'comment': 'PE ProductVersion. '},
    'FileItem/PEInfo/VersionInfoList/VersionInfoItem/FileDescription': {'type': 'comment', 'comment': 'PE FileDescription .'},
    'FileItem/PEInfo/ImportedModules/Module/ImportedFunctions/string': {'type': 'pattern-in-file', 'comment': 'PE ImportedModules. '},
    'FileItem/PEInfo/VersionInfoList/VersionInfoItem/OriginalFilename': {'type': 'pattern-in-file', 'comment': 'OriginalFilename of PE. '},

    'FormHistoryItem/HostName': {'type': 'hostname', 'comment': 'FormHistory. '},

    'Network/URI': {'type': 'uri'},
    'Network/DNS': {'type': 'domain'},
    'Network/String': {'type': 'url'},
    'Network/IPRange': {'type': 'ip-dst'},
    'Network/UserAgent': {'type': 'user-agent'},

    'PortItem/localIP': {'type': 'ip-src'},
    'PortItem/remoteIP': {'type': 'ip-dst'},
    'PortItem/remotePort': {'type': 'pattern-in-traffic', 'comment': 'RemotePort. '},

    'ProcessItem/name': {'type': 'pattern-in-memory', 'comment': 'ProcessName. '},
    'ProcessItem/path': {'type': 'pattern-in-memory', 'comment': 'ProcessPath. '},
    'ProcessItem/Mutex': {'type': 'mutex', 'comment': 'mutex'},
    'ProcessItem/arguments': {'type': 'pattern-in-memory', 'comment': 'ProcessArguments. '},
    'ProcessItem/NamedPipe': {'type': 'named pipe'},
    'ProcessItem/Pipe/Name': {'type': 'named pipe'},
    'ProcessItem/Mutex/Name': {'type': 'mutex', 'comment': 'MutexName. '},
    'ProcessItem/Event/Name': {'type': 'pattern-in-memory', 'comment': 'ProcessEventName. '},
    'ProcessItem/StringList/string': {'type': 'pattern-in-memory', 'comment': 'StringlistName. '},
    'ProcessItem/HandleList/Handle/Name': {'type': 'pattern-in-memory', 'comment': 'ProcessHandleListName'},
    'ProcessItem/HandleList/Handle/Type': {'type': 'pattern-in-memory', 'comment': 'ProcessHandleType'},
    'ProcessItem/SectionList/MemorySection/Name': {'type': 'pattern-in-memory', 'comment': 'ProcessSectionMemoryName'},
    'ProcessItem/SectionList/MemorySection/PEInfo/Exports/DllName': {'type': 'pattern-in-memory', 'comment': 'ProcessMemoryPEExportsDllName'},
    'ProcessItem/SectionList/MemorySection/PEInfo/Sections/Section/Name': {'type': 'pattern-in-memory', 'comment': 'Section name from PE in process memory section'},

    'RegistryItem/Text': {'type': 'regkey', 'comment': 'RegistryText. '},
    'RegistryItem/Path': {'type': 'regkey', 'comment': 'RegistryPath. '},
    'RegistryItem/Value': {'type': 'regkey', 'comment': 'RegistryValue. '},
    'RegistryItem/KeyPath': {'type': 'regkey', 'comment': 'RegistryKeyPath. '},
    'RegistryItem/ValueName': {'type': 'regkey', 'comment': 'RegistryValueName. '},

    'RouteEntryItem/Destination': {'type': 'ip-dst'},
    'RouteEntryItem/Destination/IP': {'type': 'ip-dst', 'comment': 'RouteDestination. '},
    'RouteEntryItem/Destination/string': {'type': 'hostname', 'comment': 'RouteDestination. '},


    'ServiceItem/name': {'type': 'windows-service-name'},
    'ServiceItem/type': {'type': 'pattern-in-memory', 'comment': 'ServiceType. '},
    'ServiceItem/startedAs': {'type': 'pattern-in-memory', 'comment': 'ServiceStartedAs. '},
    'ServiceItem/serviceDLL': {'type': 'pattern-in-memory', 'comment': 'ServiceDll. '},
    'ServiceItem/description': {'type': 'comment', 'comment': 'ServiceDescription. '},
    'ServiceItem/descriptiveName': {'type': 'windows-service-displayname'},
    'ServiceItem/serviceDLLmd5sum': {'type': 'md5', 'comment': 'ServiceDLL. '},
    'ServiceItem/serviceDLLsha1sum': {'type': 'sha1', 'comment': 'ServiceDLL. '},
    'ServiceItem/serviceDLLsha256sum': {'type': 'sha256', 'comment': 'ServiceDLL. '},
    'ServiceItem/serviceDLLSignatureVerified': {'type': 'pattern-in-memory', 'comment': 'ServiceDllSignatureVerified. '},

    'Snort/Snort': {'type': 'snort'},

    'SystemInfoItem/HostName': {'type': 'hostname', 'comment': 'SystemInfo. '},

    'TaskItem/Name': {'type': 'windows-scheduled-task', 'comment': 'TaskName. '},
    'TaskItem/sha1sum': {'type': 'windows-scheduled-task', 'comment': 'TashSha1. '},
    'TaskItem/sha256sum': {'type': 'windows-scheduled-task', 'comment': 'TashSha256. '},
    'TaskItem/AccountName': {'type': 'windows-scheduled-task', 'comment': 'TaskAccountName'},
    'TaskItem/ActionList/Action/ExecProgramPath': {'type': 'windows-scheduled-task', 'comment': 'TaskExecProgramPath. '},
    'TaskItem/TriggerList/Trigger/TriggerFrequency': {'type': 'windows-scheduled-task', 'comment': 'TaskTriggerFrequency. '},

    'UrlHistoryItem/URL': {'type': 'url', 'comment': 'UrlHistory. '},
    'UrlHistoryItem/HostName': {'type': 'hostname', 'comment': 'UrlHistory. '},

    'Yara/Yara': {'type': 'yara'}
}

iocMispCompositeMapping = {
    # mapping for composite object
    # maybe later filename|sizeinbyte
    'FileItem/FileName|FileItem/Md5sum': {'type': 'filename|md5'},
    'FileItem/FileName|FileItem/Sha1sum': {'type': 'filename|sha1'},
    'FileItem/FileName|FileItem/Sha256sum': {'type': 'filename|sha256'},
    'Network/DNS|PortItem/remoteIP': {'type': 'domain|ip'},
    'PortItem/remoteIP|PortItem/remotePort': {'type': 'ip-dst|port'},
    'RegistryItem/Path|RegistryItem/Value': {'type': 'regkey|value'},
    'RegistryItem/KeyPath|RegistryItem/Value': {'type': 'regkey|value'},
    'RegistryItem/Path|RegistryItem/Text': {'type': 'regkey|value'}
}


def extract_field(report, field_name):
    if report:
        data = report.find(field_name.lower())
        if data and hasattr(data, 'text'):
            return data.text
    return ''


def load_openioc_file(openioc_path):
    if not os.path.exists(openioc_path):
        raise Exception("Path doesn't exists.")
    with open(openioc_path, 'r') as f:
        return load_openioc(f)


def load_openioc(openioc):
    # Takes a opened file, or a string
    if not has_bs4:
        raise Exception('You need to install BeautifulSoup: pip install bs4')
    misp_event = MISPEvent()
    iocreport = BeautifulSoup(openioc, "html.parser")
    # Set event fields
    info = extract_field(iocreport, 'short_description')
    if info:
        misp_event.info = info
    date = extract_field(iocreport, 'authored_date')
    if date:
        misp_event.set_date(date)
    # Set special attributes
    description = extract_field(iocreport, 'description')
    if description:
        if not misp_event.info:
            misp_event.info = description
        else:
            misp_event.add_attribute('comment', description)
    if not misp_event.info:
        misp_event.info = 'OpenIOC import'
    author = extract_field(iocreport, 'authored_by')
    if author:
        misp_event.add_attribute('comment', author)
    misp_event = set_all_attributes(iocreport, misp_event)
    return misp_event


def get_mapping(openioc_type, mappingDict=iocMispMapping):
    t = openioc_type.lower()
    for k, v in mappingDict.items():
        if k.lower() == t:
            return v
    return False


def set_values(value1, value2=None):
    attribute_values = {}

    if value2 is not None:
        # construct attribut composite value
        value = "{}|{}".format(extract_field(value1, 'Content'),
                               extract_field(value2, 'Content')
                               )
    else:
        value = extract_field(value1, 'Content')

    if value:
        attribute_values['value'] = value
    else:
        return None

    if value2 is not None:
        # construct attribut composite type
        compositeMapping = '{}|{}'.format(value1.find('context')['search'], value2.find('context')['search'])
        mapping = get_mapping(compositeMapping, mappingDict=iocMispCompositeMapping)
    else:
        context_search = value1.find('context')['search']
        content_type = value1.find('content').get('type', None)
        if "RouteEntryItem/Destination" in context_search and content_type:
            mapping = get_mapping(context_search + '/' + content_type)
        else:
            mapping = get_mapping(context_search)

    if mapping:
        attribute_values.update(mapping)
    else:
        # Unknown mapping, assign to default
        attribute_values['category'] = 'External analysis'
        attribute_values['type'] = 'other'

    # change value to composite
    # 127.0.0.1:80 ip-* to 127.0.0.1|80 ip-*|port
    if attribute_values['type'] in ['ip-src', 'ip-dst'] and attribute_values['value'].count(':') == 1:
        attribute_values['type'] = attribute_values['type'] + '|port'
        attribute_values['value'] = attribute_values['value'].replace(':', '|')

    attribute_values["comment"] = '{}{}'.format(extract_field(value1, 'Comment'),
                                                extract_field(value2, 'Comment')
                                                )

    return attribute_values


def set_all_attributes(openioc, misp_event):
    processed = set()

    # check for composite item
    for composite in openioc.find_all("indicator", operator="AND"):

        # check for composite number under
        childs = composite.find_all('indicatoritem')

        if len(childs) == 2:
            childList = [child.find('context')['search'] for child in childs]

            def check_and_add(value1, value2):
                if (value1 and value2) in childList:
                    if childs[0].find('context')['search'] == value1:
                        attribute_values = set_values(childs[0], childs[1])
                    else:
                        attribute_values = set_values(childs[1], childs[0])

                    misp_event.add_attribute(**attribute_values)
                    processed.add(childs[0]['id'])
                    processed.add(childs[1]['id'])

            for k in iocMispCompositeMapping:
                check_and_add(k.split('|')[0], k.split('|')[1])

    for item in openioc.find_all("indicatoritem"):
        # check if id in processed list
        if item['id'] in processed:
            continue
        attribute_values = set_values(item)

        if attribute_values is None:
            continue

        misp_event.add_attribute(**attribute_values)

    return misp_event



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from .abstractgenerator import AbstractMISPObjectGenerator
import logging

logger = logging.getLogger('pymisp')


class DomainIPObject(AbstractMISPObjectGenerator):

    def __init__(self, parameters: dict, strict: bool = True, **kwargs):
        super().__init__('domain-ip', strict=strict, **kwargs)
        self._parameters = parameters
        self.generate_attributes()

    def generate_attributes(self):
        first = self._sanitize_timestamp(self._parameters.pop('first-seen', None))
        self._parameters['first-seen'] = first
        last = self._sanitize_timestamp(self._parameters.pop('last-seen', None))
        self._parameters['last-seen'] = last
        super().generate_attributes()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from .abstractgenerator import AbstractMISPObjectGenerator
import logging
from urllib.parse import unquote_plus

try:
    from pyfaup.faup import Faup  # type: ignore
except (OSError, ImportError):
    from ._psl_faup import PSLFaup as Faup

logger = logging.getLogger('pymisp')

faup = Faup()


class URLObject(AbstractMISPObjectGenerator):

    def __init__(self, url: str, generate_all=False, **kwargs):
        super().__init__('url', **kwargs)
        self._generate_all = True if generate_all is True else False
        faup.decode(unquote_plus(url))
        self.generate_attributes()

    def generate_attributes(self):
        self.add_attribute('url', value=faup.url.decode())
        if faup.get_host():
            self.add_attribute('host', value=faup.get_host())
        if faup.get_domain():
            self.add_attribute('domain', value=faup.get_domain())
        if self._generate_all:
            if hasattr(faup, 'ip_as_host') and faup.ip_as_host:
                self.attributes = [attr for attr in self.attributes
                                   if attr.object_relation not in ('host', 'domain')]
                self.add_attribute('ip', value=faup.ip_as_host)
            if faup.get_credential():
                self.add_attribute('credential', value=faup.get_credential())
            if faup.get_fragment():
                self.add_attribute('fragment', value=faup.get_fragment())
            if faup.get_port():
                self.add_attribute('port', value=faup.get_port())
            if faup.get_query_string():
                self.add_attribute('query_string', value=faup.get_query_string())
            if faup.get_resource_path():
                self.add_attribute('resource_path', value=faup.get_resource_path())
            if faup.get_scheme():
                self.add_attribute('scheme', value=faup.get_scheme())
            if faup.get_tld():
                self.add_attribute('tld', value=faup.get_tld())
            if faup.get_domain_without_tld():
                self.add_attribute('domain_without_tld', value=faup.get_domain_without_tld())
            if faup.get_subdomain():
                self.add_attribute('subdomain', value=faup.get_subdomain())
            if hasattr(faup, 'get_unicode_host') and faup.get_unicode_host() != faup.get_host():
                self.add_attribute('text', value=faup.get_unicode_host())



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from ..exceptions import InvalidMISPObject
from .abstractgenerator import AbstractMISPObjectGenerator
from io import BytesIO
from hashlib import md5, sha1, sha256, sha512
import logging
from typing import Optional, Union
from pathlib import Path
from . import FileObject

import lief  # type: ignore

try:
    import pydeep  # type: ignore
    HAS_PYDEEP = True
except ImportError:
    HAS_PYDEEP = False

logger = logging.getLogger('pymisp')


def make_macho_objects(lief_parsed: lief.Binary, misp_file: FileObject, standalone: bool = True, default_attributes_parameters: dict = {}):
    macho_object = MachOObject(parsed=lief_parsed, standalone=standalone, default_attributes_parameters=default_attributes_parameters)
    misp_file.add_reference(macho_object.uuid, 'includes', 'MachO indicators')
    macho_sections = []
    for s in macho_object.sections:
        macho_sections.append(s)
    return misp_file, macho_object, macho_sections


class MachOObject(AbstractMISPObjectGenerator):

    def __init__(self, parsed: Optional[lief.MachO.Binary] = None, filepath: Optional[Union[Path, str]] = None, pseudofile: Optional[BytesIO] = None, **kwargs):
        """Creates an MachO object, with lief"""
        super().__init__('macho', **kwargs)
        if not HAS_PYDEEP:
            logger.warning("Please install pydeep: pip install git+https://github.com/kbandla/pydeep.git")
        if pseudofile:
            if isinstance(pseudofile, BytesIO):
                self.__macho = lief.MachO.parse(raw=pseudofile.getvalue())
            elif isinstance(pseudofile, bytes):
                self.__macho = lief.MachO.parse(raw=pseudofile)
            else:
                raise InvalidMISPObject('Pseudo file can be BytesIO or bytes got {}'.format(type(pseudofile)))
        elif filepath:
            self.__macho = lief.MachO.parse(filepath)
        elif parsed:
            # Got an already parsed blob
            if isinstance(parsed, lief.MachO.Binary):
                self.__macho = parsed
            else:
                raise InvalidMISPObject('Not a lief.MachO.Binary: {}'.format(type(parsed)))
        self.generate_attributes()

    def generate_attributes(self):
        self.add_attribute('type', value=str(self.__macho.header.file_type).split('.')[1])
        self.add_attribute('name', value=self.__macho.name)
        # General information
        if self.__macho.has_entrypoint:
            self.add_attribute('entrypoint-address', value=self.__macho.entrypoint)
        # Sections
        self.sections = []
        if self.__macho.sections:
            pos = 0
            for section in self.__macho.sections:
                s = MachOSectionObject(section, standalone=self._standalone, default_attributes_parameters=self._default_attributes_parameters)
                self.add_reference(s.uuid, 'includes', 'Section {} of MachO'.format(pos))
                pos += 1
                self.sections.append(s)
        self.add_attribute('number-sections', value=len(self.sections))


class MachOSectionObject(AbstractMISPObjectGenerator):

    def __init__(self, section: lief.MachO.Section, **kwargs):
        """Creates an MachO Section object. Object generated by MachOObject."""
        # Python3 way
        # super().__init__('pe-section')
        super(MachOSectionObject, self).__init__('macho-section', **kwargs)
        self.__section = section
        self.__data = bytes(self.__section.content)
        self.generate_attributes()

    def generate_attributes(self):
        self.add_attribute('name', value=self.__section.name)
        size = self.add_attribute('size-in-bytes', value=self.__section.size)
        if int(size.value) > 0:
            self.add_attribute('entropy', value=self.__section.entropy)
            self.add_attribute('md5', value=md5(self.__data).hexdigest())
            self.add_attribute('sha1', value=sha1(self.__data).hexdigest())
            self.add_attribute('sha256', value=sha256(self.__data).hexdigest())
            self.add_attribute('sha512', value=sha512(self.__data).hexdigest())
            if HAS_PYDEEP:
                self.add_attribute('ssdeep', value=pydeep.hash_buf(self.__data).decode())
